"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.X25519 = exports.Ed25519 = exports.P256 = void 0;
const crypto_1 = require("crypto");
const P256_1 = require("./utils/P256");
const Ed25519_1 = require("./utils/Ed25519");
const X25519_1 = require("./utils/X25519");
const hkdf_1 = require("./utils/funcs/hkdf");
const symmetricKeyHash = "sha512";
const symmetricKeySaltLength = 64;
const symmetricKeyLength = 32;
const ivLength = 12;
const authTagLength = 16;
function signMessage(message, privateKey) {
    return new Promise((resolve, reject) => {
        try {
            let messageData;
            if (typeof message === "string") {
                messageData = Buffer.from(message, "utf8");
            }
            else {
                messageData = message;
            }
            const messageSignature = crypto_1.sign(null, messageData, privateKey);
            const messageSignatureBase64 = messageSignature.toString("base64");
            resolve(messageSignatureBase64);
        }
        catch (error) {
            reject(error);
        }
    });
}
function verifySignature(message, signature, publicKey) {
    return new Promise((resolve, reject) => {
        try {
            let messageData;
            if (typeof message === "string") {
                messageData = Buffer.from(message, "utf8");
            }
            else {
                messageData = message;
            }
            let signatureData;
            if (typeof signature === "string") {
                signatureData = Buffer.from(signature, "base64");
            }
            else {
                signatureData = signature;
            }
            const verification = crypto_1.verify(null, messageData, publicKey, signatureData);
            resolve(verification);
        }
        catch (error) {
            reject(error);
        }
    });
}
function generateSymmetricKey(privateKey, publicKey, salt) {
    return new Promise((resolve, reject) => {
        try {
            const sharedSecret = crypto_1.diffieHellman({ privateKey, publicKey });
            let symmetricKeySalt;
            if (salt) {
                if (Buffer.isBuffer(salt)) {
                    symmetricKeySalt = salt;
                }
                else if (typeof salt === "string") {
                    symmetricKeySalt = Buffer.from(salt, "base64");
                }
                else {
                    throw new Error("Invalid symmetric key salt");
                }
            }
            else {
                symmetricKeySalt = crypto_1.randomBytes(symmetricKeySaltLength);
            }
            const symmetricKey = hkdf_1.hkdf(sharedSecret, symmetricKeyLength, symmetricKeySalt, "", symmetricKeyHash);
            resolve({ key: symmetricKey, salt: symmetricKeySalt });
        }
        catch (error) {
            reject(error);
        }
    });
}
function encryptWithSymmetricKey(message, privateKey, publicKey) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            const symmetricKey = yield generateSymmetricKey(privateKey, publicKey);
            const symmetricKeySaltBase64 = symmetricKey.salt.toString("base64");
            const iv = crypto_1.randomBytes(ivLength);
            const cipher = crypto_1.createCipheriv("chacha20-poly1305", symmetricKey.key, iv, {
                authTagLength,
            });
            const encryptedBuffer = Buffer.concat([cipher.update(message, "utf8"), cipher.final()]);
            const tag = cipher.getAuthTag();
            const encryptedMessage = Buffer.concat([iv, encryptedBuffer, tag]);
            const encryptedMessageBase64 = encryptedMessage.toString("base64");
            resolve({ message: encryptedMessageBase64, symmetricKeySalt: symmetricKeySaltBase64 });
        }
        catch (error) {
            reject(error);
        }
    }));
}
function decryptWithSymmetricKey(encryptedMessage, privateKey, publicKey, symmetricKeySalt) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            let encryptedMessageData;
            if (typeof encryptedMessage === "string") {
                encryptedMessageData = Buffer.from(encryptedMessage, "base64");
            }
            else {
                encryptedMessageData = encryptedMessage;
            }
            const symmetricKey = yield generateSymmetricKey(privateKey, publicKey, symmetricKeySalt);
            const encryptedMessageDataLength = encryptedMessageData.length;
            const iv = encryptedMessageData.slice(0, ivLength);
            const encryptedText = encryptedMessageData.slice(ivLength, encryptedMessageDataLength - authTagLength);
            const tag = encryptedMessageData.slice(encryptedMessageDataLength - authTagLength, encryptedMessageDataLength);
            const decipher = crypto_1.createDecipheriv("chacha20-poly1305", symmetricKey.key, iv, {
                authTagLength,
            });
            decipher.setAuthTag(tag);
            const decrypted = decipher.update(encryptedText, "binary", "utf8") + decipher.final("utf8");
            resolve(decrypted);
        }
        catch (error) {
            reject(error);
        }
    }));
}
exports.P256 = {
    generateKeys: P256_1.generateP256Keys,
    loadPrivateKey: P256_1.loadP256PrivateKey,
    loadPublicKey: P256_1.loadP256PublicKey,
    formatPublicKeyToRaw: P256_1.convertP256PublicKeyToRaw,
    sign: signMessage,
    verify: verifySignature,
    encrypt: encryptWithSymmetricKey,
    decrypt: decryptWithSymmetricKey,
};
exports.Ed25519 = {
    generateKeys: Ed25519_1.generateEd25519Keys,
    loadPrivateKey: Ed25519_1.loadEd25519PrivateKey,
    loadPublicKey: Ed25519_1.loadEd25519PublicKey,
    formatPublicKeyToRaw: Ed25519_1.convertEd25519PublicKeyToRaw,
    sign: signMessage,
    verify: verifySignature,
};
exports.X25519 = {
    generateKeys: X25519_1.generateX25519Keys,
    loadPrivateKey: X25519_1.loadX25519PrivateKey,
    loadPublicKey: X25519_1.loadX25519PublicKey,
    formatPublicKeyToRaw: X25519_1.convertX25519PublicKeyToRaw,
    encrypt: encryptWithSymmetricKey,
    decrypt: decryptWithSymmetricKey,
};
const cryptokit = {
    P256: exports.P256,
    Ed25519: exports.Ed25519,
    X25519: exports.X25519,
};
exports.default = cryptokit;
