"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertEd25519PublicKeyToRaw = exports.loadEd25519PublicKey = exports.loadEd25519PrivateKey = exports.generateEd25519Keys = void 0;
const crypto_1 = require("crypto");
const hkdf_1 = require("./funcs/hkdf");
const convertToPEM_1 = require("./funcs/convertToPEM");
const algorithm = "aes-256-gcm";
const hash = "sha512";
const saltLength = 64;
const symmetricKeyLength = 32;
const ivLength = 16;
const authTagLength = 16;
const Ed25519OIDHeaderLength = 12;
const Ed25519OIDHeader = new Uint8Array([0x30, 0x2a, 0x30, 0x05, 0x06, 0x03, 0x2b, 0x65, 0x70, 0x03, 0x21, 0x00]);
const Ed25519ASNBuffer = Buffer.from(Ed25519OIDHeader, Ed25519OIDHeaderLength);
function generateEd25519Keys(password) {
    return new Promise((resolve, reject) => {
        try {
            crypto_1.generateKeyPair("ed25519", {
                publicKeyEncoding: {
                    type: "spki",
                    format: "pem",
                },
                privateKeyEncoding: {
                    type: "pkcs8",
                    format: "pem",
                },
            }, (err, publicKey, privateKey) => {
                if (err)
                    reject(err);
                const salt = crypto_1.randomBytes(saltLength);
                const keyBuffer = Buffer.from(password ? password : process.env.ED25519_PASS, "utf8");
                const key = hkdf_1.hkdf(keyBuffer, symmetricKeyLength, salt, "", hash);
                const iv = crypto_1.randomBytes(ivLength);
                const cipher = crypto_1.createCipheriv(algorithm, key, iv, { authTagLength });
                const privateKeyBufferFinal = Buffer.concat([cipher.update(privateKey, "utf8"), cipher.final()]);
                const authTag = cipher.getAuthTag();
                const privateKeyEncrypted = Buffer.concat([salt, iv, privateKeyBufferFinal, authTag]);
                const privateKeyEncryptedBase64 = privateKeyEncrypted.toString("base64");
                resolve({ publicKey, privateKey: privateKeyEncryptedBase64 });
            });
        }
        catch (error) {
            reject(error);
        }
    });
}
exports.generateEd25519Keys = generateEd25519Keys;
function loadEd25519PrivateKey(content, password) {
    return new Promise((resolve, reject) => {
        try {
            const contentBuffer = Buffer.from(content, "base64");
            const contentBufferLength = contentBuffer.length;
            const salt = contentBuffer.slice(0, saltLength);
            const iv = contentBuffer.slice(saltLength, saltLength + ivLength);
            const encryptedContent = contentBuffer.slice(saltLength + ivLength, contentBufferLength - authTagLength);
            const authTag = contentBuffer.slice(contentBufferLength - authTagLength, contentBufferLength);
            const keyBuffer = Buffer.from(password ? password : process.env.ED25519_PASS, "utf8");
            const key = hkdf_1.hkdf(keyBuffer, symmetricKeyLength, salt, "", hash);
            const decipher = crypto_1.createDecipheriv(algorithm, key, iv, { authTagLength });
            decipher.setAuthTag(authTag);
            const privateKeyDecrypted = Buffer.concat([decipher.update(encryptedContent), decipher.final()]);
            const privateKeyObject = crypto_1.createPrivateKey({ key: privateKeyDecrypted });
            resolve(privateKeyObject);
        }
        catch (error) {
            reject(error);
        }
    });
}
exports.loadEd25519PrivateKey = loadEd25519PrivateKey;
function loadEd25519PublicKey(content) {
    return new Promise((resolve, reject) => {
        try {
            const isPEM = content.includes("-----BEGIN PUBLIC KEY-----") && content.includes("-----END PUBLIC KEY-----");
            let publicKey = content;
            if (!isPEM) {
                const publicKeyBuffer = Buffer.from(content, "base64");
                if (publicKeyBuffer.length === 44) {
                    publicKey = convertToPEM_1.convertToPEM(content);
                }
                else if (publicKeyBuffer.length === 32) {
                    const publicKeyWithHeader = Buffer.concat([Ed25519ASNBuffer, publicKeyBuffer]);
                    const publicKeyWithHeaderBase64 = publicKeyWithHeader.toString("base64");
                    publicKey = convertToPEM_1.convertToPEM(publicKeyWithHeaderBase64);
                }
                else {
                    reject(new Error("Invalid Ed25519 public key length"));
                }
            }
            const publicKeyObject = crypto_1.createPublicKey({ key: publicKey });
            resolve(publicKeyObject);
        }
        catch (error) {
            reject(error);
        }
    });
}
exports.loadEd25519PublicKey = loadEd25519PublicKey;
function convertEd25519PublicKeyToRaw(publicKey) {
    return new Promise((resolve, reject) => {
        try {
            const rawWithHeader = publicKey.export({ type: "spki", format: "der" });
            const rawWithoutHeader = rawWithHeader.slice(Ed25519OIDHeaderLength);
            const rawPublicKey = rawWithoutHeader.toString("base64");
            resolve(rawPublicKey);
        }
        catch (error) {
            reject(error);
        }
    });
}
exports.convertEd25519PublicKeyToRaw = convertEd25519PublicKeyToRaw;
