"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertX25519PublicKeyToRaw = exports.loadX25519PublicKey = exports.loadX25519PrivateKey = exports.generateX25519Keys = void 0;
const crypto_1 = require("crypto");
// import { generateX25519 } from "./funcs/generateX25519";
const hkdf_1 = require("./funcs/hkdf");
const convertToPEM_1 = require("./funcs/convertToPEM");
const algorithm = "aes-256-gcm";
const hash = "sha512";
const saltLength = 64;
const symmetricKeyLength = 32;
const ivLength = 16;
const authTagLength = 16;
const X25519OIDHeaderLength = 12;
const X25519OIDHeader = new Uint8Array([0x30, 0x2a, 0x30, 0x05, 0x06, 0x03, 0x2b, 0x65, 0x6e, 0x03, 0x21, 0x00]);
const X25519ASNBuffer = Buffer.from(X25519OIDHeader, X25519OIDHeaderLength);
function generateX25519Keys(password) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            crypto_1.generateKeyPair("x25519", {
                publicKeyEncoding: {
                    type: "spki",
                    format: "pem",
                },
                privateKeyEncoding: {
                    type: "pkcs8",
                    format: "pem",
                },
            }, (err, publicKey, privateKey) => {
                if (err)
                    reject(err);
                const salt = crypto_1.randomBytes(saltLength);
                const keyBuffer = Buffer.from(password ? password : process.env.X25519_PASS, "utf8");
                const key = hkdf_1.hkdf(keyBuffer, symmetricKeyLength, salt, "", hash);
                const iv = crypto_1.randomBytes(ivLength);
                const cipher = crypto_1.createCipheriv(algorithm, key, iv, { authTagLength });
                const privateKeyBufferFinal = Buffer.concat([cipher.update(privateKey, "utf8"), cipher.final()]);
                const authTag = cipher.getAuthTag();
                const privateKeyEncrypted = Buffer.concat([salt, iv, privateKeyBufferFinal, authTag]);
                const privateKeyEncryptedBase64 = privateKeyEncrypted.toString("base64");
                resolve({ publicKey, privateKey: privateKeyEncryptedBase64 });
            });
        }
        catch (error) {
            reject(error);
        }
    }));
}
exports.generateX25519Keys = generateX25519Keys;
function loadX25519PrivateKey(content, password) {
    return new Promise((resolve, reject) => {
        try {
            const contentBuffer = Buffer.from(content, "base64");
            const contentBufferLength = contentBuffer.length;
            const salt = contentBuffer.slice(0, saltLength);
            const iv = contentBuffer.slice(saltLength, saltLength + ivLength);
            const encryptedContent = contentBuffer.slice(saltLength + ivLength, contentBufferLength - authTagLength);
            const authTag = contentBuffer.slice(contentBufferLength - authTagLength, contentBufferLength);
            const keyBuffer = Buffer.from(password ? password : process.env.X25519_PASS, "utf8");
            const key = hkdf_1.hkdf(keyBuffer, symmetricKeyLength, salt, "", hash);
            const decipher = crypto_1.createDecipheriv(algorithm, key, iv, { authTagLength });
            decipher.setAuthTag(authTag);
            const privateKeyDecrypted = Buffer.concat([decipher.update(encryptedContent), decipher.final()]);
            const privateKeyObject = crypto_1.createPrivateKey({ key: privateKeyDecrypted });
            resolve(privateKeyObject);
        }
        catch (error) {
            reject(error);
        }
    });
}
exports.loadX25519PrivateKey = loadX25519PrivateKey;
function loadX25519PublicKey(content) {
    return new Promise((resolve, reject) => {
        try {
            const isPEM = content.includes("-----BEGIN PUBLIC KEY-----") && content.includes("-----END PUBLIC KEY-----");
            let publicKey = content;
            if (!isPEM) {
                const publicKeyBuffer = Buffer.from(content, "base64");
                if (publicKeyBuffer.length === 44) {
                    publicKey = convertToPEM_1.convertToPEM(content);
                }
                else if (publicKeyBuffer.length === 32) {
                    const publicKeyWithHeader = Buffer.concat([X25519ASNBuffer, publicKeyBuffer]);
                    const publicKeyWithHeaderBase64 = publicKeyWithHeader.toString("base64");
                    publicKey = convertToPEM_1.convertToPEM(publicKeyWithHeaderBase64);
                }
                else {
                    reject(new Error("Invalid X25519 public key length"));
                }
            }
            const publicKeyObject = crypto_1.createPublicKey({ key: publicKey });
            resolve(publicKeyObject);
        }
        catch (error) {
            reject(error);
        }
    });
}
exports.loadX25519PublicKey = loadX25519PublicKey;
function convertX25519PublicKeyToRaw(publicKey) {
    return new Promise((resolve, reject) => {
        try {
            const rawWithHeader = publicKey.export({ type: "spki", format: "der" });
            const rawWithoutHeader = rawWithHeader.slice(X25519OIDHeaderLength);
            const rawPublicKey = rawWithoutHeader.toString("base64");
            resolve(rawPublicKey);
        }
        catch (error) {
            reject(error);
        }
    });
}
exports.convertX25519PublicKeyToRaw = convertX25519PublicKeyToRaw;
